@@ -1,5 +1,5 @@
 from __future__ import absolute_import
-# Copyright (c) 2010-2017 openpyxl
+# Copyright (c) 2010-2018 openpyxl
 
 #stdlib
 from io import BytesIO
@@ -11,9 +11,7 @@ from openpyxl.tests.helper import compare_xml
 
 # package
 from openpyxl import Workbook, load_workbook
-from openpyxl.workbook.defined_name import DefinedName
-from openpyxl.xml.functions import Element, tostring, fromstring
-from openpyxl.xml.constants import XLTX, XLSX, XLSM, XLTM
+from openpyxl.xml.functions import tostring
 from .. excel import (
     save_workbook,
     save_virtual_workbook,
@@ -28,7 +26,7 @@ def test_write_auto_filter(datadir):
     datadir.chdir()
     wb = Workbook()
     ws = wb.active
-    ws.cell('F42').value = 'hello'
+    ws['F42'].value = 'hello'
     ws.auto_filter.ref = 'A1:F1'
 
     content = write_workbook(wb)
@@ -46,6 +44,7 @@ def test_write_hidden_worksheet():
     expected = """
     <workbook xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships">
     <workbookPr/>
+    <workbookProtection/>
     <bookViews>
       <workbookView activeTab="1"/>
     </bookViews>
@@ -136,6 +135,7 @@ def test_write_workbook_code_name():
     expected = """
     <workbook xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships">
     <workbookPr codeName="MyWB"/>
+    <workbookProtection/>
     <bookViews>
       <workbookView activeTab="0"/>
     </bookViews>
@@ -166,6 +166,20 @@ def test_write_root_rels():
     assert diff is None, diff
 
 
+def test_write_workbook_protection(datadir):
+    from ...workbook.protection import WorkbookProtection
+
+    datadir.chdir()
+    wb = Workbook()
+    wb.security = WorkbookProtection(lockStructure=True)
+    wb.security.set_workbook_password('ABCD', already_hashed=True)
+
+    content = write_workbook(wb)
+    with open('workbook_protection.xml') as expected:
+        diff = compare_xml(content, expected.read())
+        assert diff is None, diff
+
+
 @pytest.fixture
 def Unicode_Workbook():
     wb = Workbook()
@@ -184,6 +198,7 @@ def test_print_area(Unicode_Workbook):
     <workbook xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships"
     xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">
     <workbookPr/>
+    <workbookProtection/>
     <bookViews>
       <workbookView activeTab="0"/>
     </bookViews>
@@ -210,6 +225,7 @@ def test_print_titles(Unicode_Workbook):
     <workbook xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships"
     xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">
     <workbookPr/>
+    <workbookProtection/>
     <bookViews>
       <workbookView activeTab="0"/>
     </bookViews>
@@ -229,7 +245,6 @@ def test_print_titles(Unicode_Workbook):
 def test_print_autofilter(Unicode_Workbook):
     wb = Unicode_Workbook
     ws = wb.active
-    from openpyxl.worksheet.filters import AutoFilter
     ws.auto_filter.ref = "A1:A10"
     ws.auto_filter.add_filter_column(0, ["Kiwi", "Apple", "Mango"])
 
@@ -239,6 +254,7 @@ def test_print_autofilter(Unicode_Workbook):
     <workbook xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships"
     xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">
     <workbookPr/>
+    <workbookProtection/>
     <bookViews>
       <workbookView activeTab="0"/>
     </bookViews>
