@@ -1,5 +1,5 @@
 from __future__ import absolute_import
-# Copyright (c) 2010-2017 openpyxl
+# Copyright (c) 2010-2018 openpyxl
 
 """Write the workbook global settings to the archive."""
 
@@ -21,7 +21,7 @@ from openpyxl.chartsheet import Chartsheet
 from openpyxl.packaging.relationship import Relationship, RelationshipList
 from openpyxl.workbook.defined_name import DefinedName
 from openpyxl.workbook.external_reference import ExternalReference
-from openpyxl.workbook.parser import ChildSheet, WorkbookPackage
+from openpyxl.workbook.parser import ChildSheet, WorkbookPackage, PivotCache
 from openpyxl.workbook.properties import CalcProperties, WorkbookProperties
 from openpyxl.workbook.views import BookView
 from openpyxl.utils.datetime import CALENDAR_MAC_1904
@@ -92,8 +92,8 @@ def write_workbook(workbook):
 
     # book views
     active = get_active_sheet(wb)
-    view = BookView(activeTab=active)
-    root.bookViews =[view]
+    wb.views[0].activeTab = active
+    root.bookViews = wb.views
 
     # worksheets
     for idx, sheet in enumerate(wb._sheets, 1):
@@ -145,7 +145,19 @@ def write_workbook(workbook):
 
     root.definedNames = defined_names
 
-    root.calcPr = CalcProperties(calcId=124519, fullCalcOnLoad=True)
+    # pivots
+    pivot_caches = set()
+    for pivot in wb._pivots:
+        if pivot.cache not in pivot_caches:
+            pivot_caches.add(pivot.cache)
+            c = PivotCache(cacheId=pivot.cacheId)
+            root.pivotCaches.append(c)
+            rel = Relationship(Type=pivot.cache.rel_type, Target=pivot.cache.path)
+            wb.rels.append(rel)
+            c.id = rel.id
+    wb._pivots = [] # reset
+
+    root.calcPr = wb.calculation
 
     return tostring(root.to_tree())
 
